//
// This file is based on PC6001V source files by Mr. Yumitaro
//

#include <stdio.h>
#include <windows.h>

#include "d88.h"

#define GETWORD(fp) ((word)(((byte)fgetc(fp))|(((byte)fgetc(fp))<<8)))
#define GETDWORD(fp) ((dword)(((byte)fgetc(fp))|(((byte)fgetc(fp))<<8)|(((byte)fgetc(fp))<<16)|(((byte)fgetc(fp))<<24)))
#define PUTWORD(data,fp) { fputc((data)&0xff,fp); fputc((data>>8)&0xff,fp); }
#define PUTDWORD(data,fp) { fputc((data)&0xff,fp); fputc((data>>8)&0xff,fp); fputc((data>>16)&0xff,fp); fputc((data>>24)&0xff,fp); }

// D88 sector information
typedef struct {
	byte c;
	byte h;
	byte r;
	byte n;
	word sec_no;
	byte density;
	byte deleted;		// 0x00:normal 0x10:deleted
	byte status;
	byte reserve[5];
	word size;			// data size of the sector
	word offset;		// offset to the next data from first sector
} D88SECTOR;
	
// D88 information
typedef struct {
	byte name[17];
	byte reserve[9];
	byte protect;		// 0x00:no  0x10:yes
	byte type;			// 0x00:2D  0x10:2DD  0x20:2HD
	dword size;			// disk size
	dword table[164];	// offset table of track part (track 0-163)
	D88SECTOR secinfo;
	FILE *fp;
} D88INFO;

static D88INFO d88;		// D88 info

// read D88 header
static int ReadHeader88(void)
{
	if (d88.fp) {
		int i;        
		fread(d88.name, sizeof(byte), 17, d88.fp);
		d88.name[16] = '\0';
		if (fread(d88.reserve, 1, 9, d88.fp)<9) return 0;
		if (feof(d88.fp)) return 0;
		d88.protect = fgetc(d88.fp);
		if (feof(d88.fp)) return 0;
		d88.type = fgetc(d88.fp);
		if (feof(d88.fp)) return 0;
		d88.size = GETDWORD(d88.fp);
		for (i=0; i<164; i++) {
			if (feof(d88.fp)) return 0;
			d88.table[i] = GETDWORD(d88.fp);
		}
		return 1;
	} else return 0;
}

// initialise
void Init88()
{
	ZeroMemory(&d88,sizeof(D88INFO));
}

// mount
int Mount88(char *fname)
{
	if (fopen_s(&d88.fp, fname, "rb")) {
		return 0;
	} 
	return ReadHeader88();
}

// unmount
void Unmount88()
{
	if (d88.fp) fclose(d88.fp);
	d88.fp = NULL;
}

// read sector information
static void ReadSector88(void)
{

	if (d88.fp) {
		d88.secinfo.c = fgetc(d88.fp);
		d88.secinfo.h = fgetc(d88.fp);
		d88.secinfo.r = fgetc(d88.fp);
		d88.secinfo.n = fgetc(d88.fp);
		d88.secinfo.sec_no = GETWORD(d88.fp);
		d88.secinfo.density = fgetc(d88.fp);
		d88.secinfo.deleted = fgetc(d88.fp);
		d88.secinfo.status = fgetc(d88.fp);
		fread(d88.secinfo.reserve, sizeof(byte), 5, d88.fp);
		d88.secinfo.size = GETWORD(d88.fp);
		d88.secinfo.offset = 0;
	}
}

// read 1 byte
byte Getc88()
{
	if (d88.fp) {
		byte dat = fgetc(d88.fp);
		d88.secinfo.offset++;
		if (d88.secinfo.offset >= d88.secinfo.size) ReadSector88();
		return dat;
	}
	return 0xff;
}

// seek
int Seek88(int trackno, int sectno)
{
	if (d88.fp) {
		int try_;
		if (!d88.table[trackno]) return 0;
		fseek(d88.fp, (long)d88.table[trackno], SEEK_SET);
		ReadSector88();
		if (d88.secinfo.sec_no < sectno) return 0;
		try_=2;
		while (d88.secinfo.r != sectno) {
			if (!try_) return 0;
			fseek(d88.fp, (long)d88.secinfo.size, SEEK_CUR);
			ReadSector88();
			if (d88.secinfo.r == d88.secinfo.sec_no) try_--;
		}
		return 1;
	}
	return 0;
}

// get CHRN
dword GetCHRN88(void)
{
	return ((d88.secinfo.c<<24) || (d88.secinfo.h<<16) || (d88.secinfo.r<<8) || (d88.secinfo.n));
}// get disk type

byte GetType(void)
{
	return d88.type;
}