/**
 *
 *  SD6031Air Web Application
 *
 *  main.js
 *
 *  by Koichi Nishida on 2016/05/11 
 *
 *  Copyright (c) 2016 Koichi Nishida
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 *  based on a FlashAir sample program
 *  by Junichi Kitano, Fixstars Corporation on 2013/05/15.
 * 
 *  Copyright (c) 2013, TOSHIBA CORPORATION
 *  All rights reserved.
 *  Released under the BSD 2-Clause license.
 *   http://flashair-developers.com/documents/license.html
 */
// JavaScript Document

iCabMobile_FileFieldActive = 1;

function getExtension(fileName) {
    var ret;
    if (!fileName) {
      return ret;
    }
    var fileTypes = fileName.split(".");
    var len = fileTypes.length;
    if (len === 0) {
      return ret;
    }
    ret = fileTypes[len - 1];
    return ret;
}

// Convert data format from V1 to V2.
function convertFileList(files) {
	for (var i = 0; i < files.length; i++) {
		var elements = files[i].split(",");
		files[i] = new Array();
		files[i]["r_uri"] = elements[0];
		files[i]["fname"] = elements[1];
		files[i]["fsize"] = Number(elements[2]);
		files[i]["attr"]  = Number(elements[3]);
		files[i]["fdate"] = Number(elements[4]);
		files[i]["ftime"] = Number(elements[5]);
	}
}

// Callback Function for sort()
function cmporder(a, b) {
	if( a["fname"] > b["fname"] ) {
		return 1;
	} else if ( a["fname"] < b["fname"] ) {
		return -1;
	} else if( a["fdate"] == b["fdate"] ) {
        return a["ftime"] - b["ftime"];
    }else{
        return a["fdate"] - b["fdate"];
    }
}

// used for filtering files
var ext = {'D88':false, 'P31':false};

// Show file list
function showFileList(files, path) {
	// Clear box.
	$("#list").html('');
	$("#list").append(
    		$("<div></div>").append(
			$('<a href="javascript:void(0)" class="file">Unmount</a>')
        	)
	);
	// Output a link to the parent directory if it is not the root directory.
	if( path != "/" ) {
		$("#list").append(
			$("<div></div>").append(
				$('<a href="javascript:void(0)" class="dir">..</a>')
			)
		);
	}
	$.each(files, function() {
		var file = this;
		// Skip hidden file.
		if ( file["attr"] & 0x02 ) {
			return;
		}
		// Make a link to directories and files.
		var filelink = $('<a href="javascript:void(0)"></a>');
		var caption = file["fname"];
		var fileobj = $("<div></div>");
		if ( file["attr"] & 0x10 ) {
			filelink.addClass("dir");
		} else {
			var e = getExtension(caption).toUpperCase();
			if (!((ext['D88']&&(e=='D88'))||(ext['P31']&&(e=='P31'))||(!ext['D88']&&!ext['P31']))) {
				return;
			}
			filelink.addClass("file");
		}
		// Append a file entry or directory to the end of the list.
		$("#list").append(
			fileobj.append(
				filelink.append(
					caption
				)
			)
		);
	});
}

//Making Path
function makePath(curPath, dir) {
	var arrPath = curPath.split('/');
	if ( curPath == "/" ) {
		arrPath.pop();
	}
	if ( dir == ".." ) {
		// Go to parent directory. Remove last fragment.
		arrPath.pop();
	} else if ( dir != "" && dir != "." ) {
		// Go to child directory. Append dir to the current path.
		arrPath.push(dir);
	}
	if ( arrPath.length == 1 ) {
		arrPath.push("");
	}
	return arrPath.join("/");
}

// Get file list
function getFileList(path) {
	// Make URL for CGI. (DIR must not end with '/' except if it is the root.)
	var url = "/command.cgi?op=100&DIR=" + path;
	// Issue CGI command.
	var files;
 	jQuery.ajax({
		url: url,
		async: false,
		success: function( data ) {
			// Save the current path.
			// Split lines by new line characters.
			files = data.split(/\n/g);
			// Ignore the first line (title) and last line (blank).
			files.shift();
			files.pop();
			// Convert to V2 format.
			convertFileList(files);
			// Sort by date and time.
			files.sort(cmporder);
		},
		error: function( data ) {
		}
	});
	return files;
}

function mkTimeString() {
	var dt = new Date();
	var year = (dt.getFullYear() - 1980) << 9;
	var month = (dt.getMonth() + 1) << 5;
	var date = dt.getDate();
	var hours = dt.getHours() << 11;
	var minites = dt.getMinutes() << 5;
	var seconds = Math.floor(dt.getSeconds() / 2);
	var timestring = "0x" + (year + month + date).toString(16) + (hours + minites + seconds).toString(16);	
	return timestring;
}

//UploadProcess
function doUpload() {
	var path = makePath(currentPath, ".");
	var cgi = "/upload.cgi";
	var timestring = mkTimeString();
	$.get(cgi + "?UPDIR=" + path + "&FTIME=" + timestring, function() {
		$('#uploadbutton').attr('value', 'Uploading!');
		var  f = document.getElementById("my_form");
		var fd = new FormData(f);
		$.ajax({ url: cgi,
			type: "POST",
			data: fd,
			processData: false,
			contentType: false,
			//mimeType: "multipart/form-data",
			success: function(html){
				if ( html.indexOf("SUCCESS") ) {
					showFileList(getFileList(currentPath), currentPath);
				}else{ // error
				}
				$('#uploadbutton').attr('value', 'Upload');
			}
		});
		return false;
	});
}

// make a directory
function doMkDir(path) {
	var timestring = mkTimeString();
	var cgi = "/upload.cgi";
	$.get(cgi + "?UPDIR=" + path + "&FTIME=" + timestring, function() {
		showFileList(getFileList(currentPath), currentPath);
	});
}

// delete a file
function doDelete(path, confirm) {
	if(!confirm || window.confirm('delete '+path+'?')){
		var url = "/upload.cgi?DEL=" + path;
		jQuery.ajax( {
    		url: url,
			async: false,
       		success: function( data ) {}
		});
	}
}

// delete a directory
function deleteDirSub(path, files) {
  $.each(files, function() {
		var file = this;
		if ( file["attr"] & 0x10 ) {
        	doDeleteDir(makePath(path, file["fname"]));
		} else {
	 		doDelete(makePath(path, file["fname"]), false);
		}
  });
}
function doDeleteDir(path) {
	if(window.confirm('delete the folder and all files in '+path+'?')){
		deleteDirSub(path, getFileList(path));
		doDelete(path, false);
	}
}

function encode(str){
	var uni = "";
	for( var i=0;i<str.length;i++){
		uni += ("000"+str.charCodeAt(i).toString(16)).slice(-4);
	}
	return uni;
}

// select an image
function selectImage(drive, file) {
	if (file == "Unmount") {
		var url = "/command.cgi?op=131&ADDR=0&LEN=2&DATA=U"+drive;
		// Issue CGI command.
		$.get(url, function() {
		});
		return;
	}
	var e = getExtension(file).toUpperCase();
	if ((e != "D88") && (e != "P31")) return;
	var data = makePath(currentPath, file+"\0");
	if ((data.split('/').length-1)>5) {
		alert("folder is too deep.");
		return;
	}
	var encoded = encode(data);
	if (encoded.length > 510) {
		alert("too long path length.");
		return;
	}
 	var url = "/command.cgi?op=131&ADDR=0&LEN="+(encoded.length+2)+"&DATA=D"+drive+encoded;
	// Issue CGI command.
	$.get(url, function() {
	});
}

function timerFunc() {
	var url = "/command.cgi?op=130&ADDR=0&LEN=64";
	$.get(url, function(data) {
		if ((data.substr(0,1)=='T') || (data.substr(0,1)=='T')) {
			var m = data.charCodeAt(1,1);
			var type_rdv = data.charCodeAt(6,1);
			pc6031prot = (type_rdv >> 4);
			var img0 = data.substr(8, 8);
			var img1 = data.substr(16, 8);
			var img2 = data.substr(24, 8);
			var img3 = data.substr(32, 8);
			$("#pc6031image0").html(img0);
			$("#pc6031image1").html(img1);
			$("#pc6031image2").html(img2);
			$("#pc6031image3").html(img3);
			$('#writeprotbutton0').attr("checked", (pc6031prot&1)?true:false);
			$('#writeprotbutton1').attr("checked", (pc6031prot&2)?true:false);
			$('#writeprotbutton2').attr("checked", (pc6031prot&4)?true:false);
			$('#writeprotbutton3').attr("checked", (pc6031prot&8)?true:false);
			$('#writeprotbutton0').attr("disabled", false);
			$('#writeprotbutton1').attr("disabled", false);
			$('#writeprotbutton2').attr("disabled", false);
			$('#writeprotbutton3').attr("disabled", false);
		}
		return false;
	});
}

//Document Ready
$(function() {
	// Iniialize global variables.
	currentPath = location.pathname;
	// Show the root directory.
	showFileList(getFileList(currentPath), currentPath);
	// Register onClick handler for <a class="dir">
	$(document).on("dblclick","a.dir",function() {
		currentPath = makePath(currentPath, this.text);
		showFileList(getFileList(currentPath), currentPath);
	});
	$("#cmdChDir").click(function(e) {
		var cur = $('#cur');
		if (cur.hasClass("dir")) {
			currentPath = makePath(currentPath, $('#cur').text());
			showFileList(getFileList(currentPath), currentPath);
		}
	});
	$(document).on("click","a.dir",function() {
		$('#cur').css("background-color","#000000");
		$('#cur').css("color","#ffffff");
		$('#cur').attr('id', '');
		$(this).css("color","#000000");
		$(this).css("background-color","#ffffff");
		$(this).attr('id', 'cur');
	}); 
	$(document).on("click","a.file",function() {
		$('#cur').css("background-color","#000000");
		$('#cur').css("color","#ffffff");
		$('#cur').attr('id', '');
		$(this).css("color","#000000");
		$(this).css("background-color","#ffffff");
		$(this).attr('id', 'cur');
		var link = makePath(currentPath, $(this).text());
		$('#download').attr('href', link);
	});
	$('#pc6031button0').click(function(e) {
		selectImage(0, $('#cur').text());
	});
	$('#pc6031button1').click(function(e) {
		selectImage(1, $('#cur').text());
	});
	$('#pc6031button2').click(function(e) {
		selectImage(2, $('#cur').text());
	});
	$('#pc6031button3').click(function(e) {
		selectImage(3, $('#cur').text());
	});
	$("#cmdMkDir").click(function(e) {
		var newdir = prompt("new folder name");
		if (newdir != "") {
 			doMkDir(makePath(currentPath, newdir));
		}
	});
	$("#cmdDelete").click(function(e) {
		var cur = $('#cur');
		if (cur.hasClass("file")) { 
			doDelete(makePath(currentPath, $('#cur').text()), true);
			showFileList(getFileList(currentPath), currentPath);
		} else if (cur.hasClass("dir")) {
			if (cur.text() != '..') {
				doDeleteDir(makePath(currentPath, cur.text()));
			}
		}
		showFileList(getFileList(currentPath), currentPath);
	});
	timer = setInterval(timerFunc, 1000);
	$("#uploadbutton").click(function(e) {
		doUpload();
		//showFileList(getFileList(currentPath), currentPath);
		return false;
	});
	$("#filterd88").click(function(e) {
		ext['D88']=this.checked;
		showFileList(getFileList(currentPath), currentPath);
	});
	$("#filterp31").click(function(e) {
		ext['P31']=this.checked;
		showFileList(getFileList(currentPath), currentPath);
	});
	
	$("#writebackbutton0").click(function(e) {
		if(window.confirm('Write back to the D88 image?')) {
			var url = "/command.cgi?op=131&ADDR=0&LEN=2&DATA=W0";
			$.get(url, function() {
			});
		}
	});
	$("#writebackbutton1").click(function(e) {
		if(window.confirm('Write back to the D88 image?')) {
			var url = "/command.cgi?op=131&ADDR=0&LEN=2&DATA=W1";
			$.get(url, function() {
			});
		}
	});
	$("#writebackbutton2").click(function(e) {
		if(window.confirm('Write back to the D88 image?')) {
			var url = "/command.cgi?op=131&ADDR=0&LEN=2&DATA=W2";
			$.get(url, function() {
			});
		}
	});
	$("#writebackbutton3").click(function(e) {
		if(window.confirm('Write back to the D88 image?')) {
			var url = "/command.cgi?op=131&ADDR=0&LEN=2&DATA=W3";
			$.get(url, function() {
			});
		}
	});
	$("#writeprotbutton0").attr("checked", false);
	$("#writeprotbutton0").click(function(){
		var v = (($(this).is(':checked'))?4:0)|0;
		var url = "/command.cgi?op=131&ADDR=0&LEN=2&DATA=P"+v;
		$.get(url, function() {
		});
		$(this).attr("disabled", true);
	});
	$("#writeprotbutton1").attr("checked", false);
	$("#writeprotbutton1").click(function(){
		var v = (($(this).is(':checked'))?4:0)|1;
		var url = "/command.cgi?op=131&ADDR=0&LEN=2&DATA=P"+v;
		$.get(url, function() {
		});
		$(this).attr("disabled", true);
	});
	$("#writeprotbutton2").attr("checked", false);
	$("#writeprotbutton2").click(function(){
		var v = (($(this).is(':checked'))?4:0)|2;
		var url = "/command.cgi?op=131&ADDR=0&LEN=2&DATA=P"+v;
		$.get(url, function() {
		});
		$(this).attr("disabled", true);
	});
	$("#writeprotbutton3").attr("checked", false);
	$("#writeprotbutton3").click(function(){
		var v = (($(this).is(':checked'))?4:0)|3;
		var url = "/command.cgi?op=131&ADDR=0&LEN=2&DATA=P"+v;
		$.get(url, function() {
		});
		$(this).attr("disabled", true);
	});
});
